/*
 * OutputBuffer.cpp
 *
 *  Created on: 05-Jul-2009
 *      Author: sri
 */

#include <ImageBuffer.h>
#include <iostream>
#include <pfm.h>

using namespace Imf;
using namespace Imath;

using std::cout;
using std::cerr;
using std::endl;

ImageBuffer::ImageBuffer(){
	canvas=NULL;
	bytesPerChannel=0;
	numChannels=0;
//	cvNamedWindow("__anim", 1);
}
ImageBuffer::~ImageBuffer(){
//	cvDestroyWindow("__anim");
	if (isValid())
		cvReleaseImage(&canvas);
}
void ImageBuffer::init(int width, int height, int channels, int bytesPerChannel){
	if (isValid()){
		cvReleaseImage(&canvas);
		canvas=NULL;
	}
	numChannels=channels;
	this->bytesPerChannel = bytesPerChannel;

	if (bytesPerChannel == 1)
		canvas=cvCreateImage(cvSize(width, height), IPL_DEPTH_8U, channels);
	else if (bytesPerChannel == 4)
		canvas=cvCreateImage(cvSize(width, height), IPL_DEPTH_32F, channels);

//	DEBUG (if(!isValid()) {cout << "Error creating image"<<endl;} else {cout<<"Created image buffer"<<endl;})
}

bool ImageBuffer::isValid(){
	return (canvas != NULL);
}

void ImageBuffer::load(string fname){
	cerr<<"Loading: "<<fname<<endl;
	if (isValid()){
		if (fname.find(".exr")!=string::npos && bytesPerChannel==4)//load an exr image
			loadFileEXR(fname.c_str());//Does Not Work
		else{
			canvas = cvLoadImage(fname.c_str());
			numChannels = canvas->nChannels;
			bytesPerChannel = canvas->depth/8;
		}
	}	
}

void ImageBuffer::save(string fname){
	cerr<<"Saving: "<<fname<<endl;
	if (isValid()){
		if (fname.find(".exr")!=string::npos && bytesPerChannel==4)//save an exr image
			saveFileEXR(fname.c_str());
		else if (fname.find(".pfm")!=string::npos && bytesPerChannel==4 && numChannels==3)//save a pfm image
			write_pfm_file3(fname.c_str(), (float *)(canvas->imageData), canvas->width, canvas->height);
		else
			cvSaveImage(fname.c_str(),canvas);
	}
}

int ImageBuffer::getWidth(){
	if (isValid())
		return canvas->width;
	else
		return 0;
}
int ImageBuffer::getHeight(){
	if (isValid())
		return canvas->height;
	else
		return 0;
}

void * ImageBuffer::getBuffer(){
	return (void *)(canvas->imageData);
}

int ImageBuffer::getNumChannels(){
	return numChannels;
}
int ImageBuffer::getBytesPerChannel(){
	return bytesPerChannel;
}

void ImageBuffer::show(string windowName){
	cvNamedWindow(windowName.c_str(), 1);
//	cvMoveWindow("Smile", 100, 100);
	cvShowImage(windowName.c_str(), canvas);

	//wait for key to close the window
	cvWaitKey(0);
	cvDestroyWindow( windowName.c_str() );
}

//Be VERY CAREFUL while calling this function. its pretty dangerous if misused
void ImageBuffer::copyFromBuffer(void * buffer){
	memcpy (canvas->imageData, buffer, canvas->width * canvas->height * numChannels * bytesPerChannel);
}

//void ImageBuffer::showAnimated(int waitMS){
//	cvShowImage("__anim",canvas);
//	cvWaitKey(waitMS);
//}

//Save to EXR half format. Courtesy: Rohit Garg (rpg314@gmail.com)
void ImageBuffer::saveFileEXR (const char fileName[]){
	const int pixel_count=canvas->width * canvas->height;
	Rgba *half_data=new Rgba[pixel_count];
	int cnt=0;
	for(int i=0; i<canvas->width; i++){
		for (int j=0; j<canvas->height; j++){
			half_data[cnt].a=1;
			half_data[cnt].r=half(pixelf(j,i,2));
			half_data[cnt].g=half(pixelf(j,i,1));
			half_data[cnt].b=half(pixelf(j,i,0));
			cnt++;//Major bug fixed... me so stupid :(
		}
	}
	RgbaOutputFile file (fileName, canvas->width, canvas->height, WRITE_RGBA);
	file.setFrameBuffer (half_data, canvas->width,1);
	file.writePixels (canvas->height);
}

void ImageBuffer::loadFileEXR (const char fname[]){
	Rgba * data;
	int width, height;
		
	//read the data
	data = NULL;
	readRgbaEXR(fname,data,width,height);
	if (data == NULL){
		cerr<<"Cannot load file"<<fname<<endl;
		return;
	}
	
	init(width, height, 3, 4);//create an image
	
	int cnt=0;
	for (int x=0;x<width; x++){
		for (int y=0;y<height;y++){
			pixelf(y,x,2) = data[cnt].r;
			pixelf(y,x,1) = data[cnt].g;
			pixelf(y,x,0) = data[cnt].b;
			cnt++;//next pixel
		}
	}
	
	delete [] data;
}

void ImageBuffer::readRgbaEXR (const char fname[], Rgba * &pixels, int &width, int &height)
{
	RgbaInputFile file (fname);
	Box2i dw = file.dataWindow();
	width = dw.max.x - dw.min.x + 1;
	height = dw.max.y - dw.min.y + 1;
	
	pixels = new Rgba[height*width];
	file.setFrameBuffer (pixels - dw.min.x - dw.min.y * width, 1, width);
	file.readPixels (dw.min.y, dw.max.y);
}


//returns a reference to the pixel
//Note: Channel 2=R, 1=G, 0=B
unsigned char& ImageBuffer::pixeli(int row, int col, int channel){
	return ((uchar*)(canvas->imageData + canvas->widthStep*row))[col*numChannels+channel] ;
}

float& ImageBuffer::pixelf(int row, int col, int channel){
	return ((float*)(canvas->imageData + canvas->widthStep*row))[col*numChannels+channel] ;
}

