#include <diffraction.h>
#include <ImageBuffer.h>

//to save a 4 channel texture as a file:
void saveTextureToFile(Texture* tex, string fname, int size)
	{
	ImageBuffer imageFile;
	imageFile.init(size,size, 3, 4);
	float * tmpBuffer = new float[size * size * 4];//4 channel texture
	tex->readBack(tmpBuffer);
	for (int i=0;i<SCREEN_SIZE;i++)
		{
		for (int j=0;j<SCREEN_SIZE;j++)//4 channel texture
			{//FIXME: The image is flipped along x and y
			imageFile.pixelf(i,j,2) = tmpBuffer[i*size*4 + j*4];
			imageFile.pixelf(i,j,1) = tmpBuffer[i*size*4 + j*4 +1];
			imageFile.pixelf(i,j,0) = tmpBuffer[i*size*4 + j*4 +2];
			}
		}
	imageFile.save(fname);
	delete [] tmpBuffer;
	}

//load a 4 channel texture from png
void loadTextureFromFile(Texture*& tex, string fname)
	{
	ImageBuffer buffer1;
	buffer1.load(fname);
	
	float * tmpBuffer = new float[buffer1.getWidth() * buffer1.getHeight() * 4];//4 channel texture

	for (int i=0;i<buffer1.getWidth();i++)
		{
		for (int j=0;j<buffer1.getHeight();j++)
			{
			for (int j=0;j<SCREEN_SIZE;j++)//4 channel texture
				{//FIXME: The image is flipped along x and y
				tmpBuffer[j*buffer1.getWidth()*4 + i*4] =	 buffer1.pixelf(j,i,2);
				tmpBuffer[j*buffer1.getWidth()*4 + i*4 +1] = buffer1.pixelf(j,i,1);
				tmpBuffer[j*buffer1.getWidth()*4 + i*4 +2] = buffer1.pixelf(j,i,0);
				}
			}
		}

	tex = new Texture(buffer1.getWidth(),buffer1.getHeight(),GL_RGBA, 0, tmpBuffer);
	tex->bind();
		tex->setFilter(GL_LINEAR);//linear filtering
		tex->setWrapMode(GL_CLAMP);//we dont go outside bounds
	tex->unbind();
	
	delete [] tmpBuffer;
	}
	
//get uniform values, set them etc:
void initializeShaderUniforms(Renderer& renderObject)
	{
//get the uniform variables:
	renderObject.Shaders[WLF_SHADER]->initUniform("bucketSize",WLF_BUCKET_SIZE_UNIFORM,1,true);
	renderObject.Shaders[WLF_SHADER]->initUniform("bucketStart",WLF_BUCKET_START_UNIFORM,2,false);
	renderObject.Shaders[WLF_SHADER]->initUniform("randomTexture",WLF_RAND_TEXTURE_UNIFORM,1,true);
	renderObject.Shaders[WLF_SHADER]->initUniform("lightFilterTexture",WLF_FILTER_TEXTURE_UNIFORM,1,true);	
	renderObject.Shaders[WLF_SHADER]->initUniform("sampleStep",WLF_SAMPLE_STEP_UNIFORM,1,false);
	renderObject.Shaders[WLF_SHADER]->initUniform("gratingSize",WLF_GRATING_SIZE_UNIFORM,1,false);	
	renderObject.Shaders[WLF_SHADER]->initUniform("randomSize",WLF_RANDOM_SIZE_UNIFORM,1,true);		
//	renderObject.Shaders[WLF_SHADER]->initUniform("intensityNorm",WLF_INTENISTY_NORM_UNIFORM,1,false);
	
	renderObject.Shaders[WLF_SHADER]->initUniform("lightSource",WLF_LIGHTSOURCE_UNIFORM,3,false);
	renderObject.Shaders[WLF_SHADER]->initUniform("lightColor",WLF_LIGHTCOLOR_UNIFORM,4,false);	
	renderObject.Shaders[WLF_SHADER]->initUniform("gratingPosition",WLF_GRATING_POSITION_UNIFORM,3,false);	
	renderObject.Shaders[WLF_SHADER]->initUniform("gratingXAxis",WLF_GRATING_XAXIS_UNIFORM,3,false);	
	renderObject.Shaders[WLF_SHADER]->initUniform("gratingYAxis",WLF_GRATING_YAXIS_UNIFORM,3,false);
	renderObject.Shaders[WLF_SHADER]->initUniform("gratingParameters",WLF_GRATING_PARAM_UNIFORM,4,false);

	renderObject.Shaders[CLEAR_SHADER]->initUniform("clearColor",CLEAR_COLOR_UNIFORM,4,false);
	renderObject.Shaders[ACCUMULATE_SHADER]->initUniform("sourceTexture1",ACCUMULATOR_TEXTURE1_UNIFORM,1,true);
	renderObject.Shaders[ACCUMULATE_SHADER]->initUniform("sourceTexture2",ACCUMULATOR_TEXTURE2_UNIFORM,1,true);
	renderObject.Shaders[ACCUMULATE_SHADER]->initUniform("coeff",ACCUMULATOR_COEFFS_UNIFORM,3,false);
	renderObject.Shaders[ACCUMULATE_SHADER]->initUniform("flags",ACCUMULATOR_FLAGS_UNIFORM,1,true);
	renderObject.Shaders[FINAL_DRAW_SHADER]->initUniform("sourceTexture",FINAL_DRAW_TEXTURE_UNIFORM,1,true);
	renderObject.Shaders[DIFFUSE_SHADER]->initUniform("light_position",DIFFUSE_LIGHT_SOURCE,3,false);
	renderObject.Shaders[DIFFUSE_SHADER]->initUniform("light_color",DIFFUSE_LIGHT_COLOR,4,false);

//assign values to the uniform variables
//	renderObject.Shaders[WLF_SHADER]->assignUniform1f(WLF_BUCKET_SIZE_UNIFORM,BUCKET_SIZE);
//	renderObject.Shaders[WLF_SHADER]->assignUniform1f(WLF_SAMPLE_STEP_UNIFORM,SAMPLE_STEP);
	renderObject.Shaders[WLF_SHADER]->assignUniform1i(WLF_RAND_TEXTURE_UNIFORM,0);
	renderObject.Shaders[WLF_SHADER]->assignUniform1i(WLF_FILTER_TEXTURE_UNIFORM,1);	
	renderObject.Shaders[WLF_SHADER]->assignUniform1i(WLF_RANDOM_SIZE_UNIFORM,RANDOM_TEXTURE_SIZE);
//	renderObject.Shaders[WLF_SHADER]->assignUniform1f(WLF_INTENISTY_NORM_UNIFORM,//reciprocal of #samples
//									(SAMPLE_STEP*SAMPLE_STEP)/(GRATING_SIZE*GRATING_SIZE));
									
	renderObject.Shaders[CLEAR_SHADER]->assignUniform4f(CLEAR_COLOR_UNIFORM,0,0,0,1);
	renderObject.Shaders[FINAL_DRAW_SHADER]->assignUniform1i(FINAL_DRAW_TEXTURE_UNIFORM,0);		
	
	renderObject.Shaders[ACCUMULATE_SHADER]->assignUniform1i(ACCUMULATOR_TEXTURE1_UNIFORM,0);
	renderObject.Shaders[ACCUMULATE_SHADER]->assignUniform1i(ACCUMULATOR_TEXTURE2_UNIFORM,1);
	renderObject.Shaders[ACCUMULATE_SHADER]->assignUniform3f(ACCUMULATOR_COEFFS_UNIFORM,1.0f,1.0f,0.0f);
	renderObject.Shaders[ACCUMULATE_SHADER]->assignUniform1i(ACCUMULATOR_FLAGS_UNIFORM,0);
	
	const float Light_Intensity = 20.0f;
	renderObject.Shaders[DIFFUSE_SHADER]->assignUniform4f(DIFFUSE_LIGHT_COLOR,Light_Intensity,Light_Intensity,Light_Intensity,1.0f);//light is off
	renderObject.Shaders[DIFFUSE_SHADER]->assignUniform3f(DIFFUSE_LIGHT_SOURCE,-10.0f,-10.0f,-10.0f);
	
	renderObject.Shaders[TONE_MAP_SHADER]->initUniform("defog",TONEMAP_DEFOG,1,false);		
	renderObject.Shaders[TONE_MAP_SHADER]->initUniform("exposure",TONEMAP_EXPOSURE,1,false);		
	renderObject.Shaders[TONE_MAP_SHADER]->initUniform("knee_low",TONEMAP_KNEE_LOW,1,false);		
	renderObject.Shaders[TONE_MAP_SHADER]->initUniform("knee_f_factor",TONEMAP_KNEE_F,1,false);
	renderObject.Shaders[TONE_MAP_SHADER]->initUniform("sourceTexture",TOMEMAP_TEXTURE,1,true);
	}

//to create a random 2d texture for storing random numbers
void setupRandomTexture(Texture*& tex, int width, int height)
	{
	float * randTex= new float[width*height*3];
	
	//generate random numbers from 0 to 1
	for (int i=0; i<width*height*3; i++)
		randTex[i]=(float)rand()/(float)RAND_MAX;
	
	tex = new Texture(width,height,GL_RGB, 0, randTex);
	tex->bind();
		tex->setFilter(GL_NEAREST);//no filtering
		tex->setWrapMode(GL_REPEAT);//we can safely go out of bounds and still get random numbers
	tex->unbind();
	
	delete [] randTex;
	}

//swap integers
void swapTargets(RenderTargetEnum& a, RenderTargetEnum& b)
	{
	RenderTargetEnum tmp=a;
	a=b;
	b=tmp;
	}

Texture* computeDiffractionTexture(Renderer& renderObject, string fname, const grating& g)
	{    
    //Clear the 2 accumulators:
    applyShader(renderObject.Shaders[CLEAR_SHADER], renderObject.RenderTargets[ACCUMULATOR1], SCREEN_SIZE);
    applyShader(renderObject.Shaders[CLEAR_SHADER], renderObject.RenderTargets[ACCUMULATOR2], SCREEN_SIZE);
    
    RenderTargetEnum SRC_ACCUMULATOR = ACCUMULATOR1;//read from this
    RenderTargetEnum DST_ACCUMULATOR = ACCUMULATOR2;//write to this
    
    float half_grating_size = g.getGratingSize()*0.5f;
    float grating_step_size = g.getBucketSize()*(float)g.getGratingSize()/(float)g.getNumSamples();
    
    for (float bucketX = -half_grating_size; bucketX < half_grating_size; bucketX += grating_step_size)
		{
		for (float bucketY = -half_grating_size; bucketY < half_grating_size; bucketY += grating_step_size)
			{
			renderObject.RenderTargets[OUTPUT_BUFFER]->enable(); //Enable the buffer onto which we render temporary results
			glClearColor(0.1,0.5,0.5, 1);
			glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
			
			renderObject.Shaders[WLF_SHADER]->bind(); //Enable the WLF shader

			//set the uniform variables for this shader
			renderObject.Shaders[WLF_SHADER]->assignUniform2f(WLF_BUCKET_START_UNIFORM,bucketX,bucketY);
			renderObject.Shaders[WLF_SHADER]->setAllUniforms();
		
			glActiveTexture(GL_TEXTURE0);//set active texture unit as 0
			renderObject.Textures[RANDOM_TEXTURE]->enable(true);//bind the texture
				
			renderObject.VertexBuffers[SCENE_VBO]->render(true);//bind and render the Scene geometry
			
			//finish up here (may not be needed inside the loop?)
			renderObject.Textures[RANDOM_TEXTURE]->disable(true);//unbind the texture
			renderObject.Shaders[WLF_SHADER]->unbind(); //disable the WLF shader
			renderObject.RenderTargets[OUTPUT_BUFFER]->disable(); 
			
			//accumulate shaded buffer output.
			applyShader(renderObject.Shaders[ACCUMULATE_SHADER], 
				renderObject.RenderTargets[SRC_ACCUMULATOR]->texture(), 
				renderObject.RenderTargets[OUTPUT_BUFFER]->texture(), 
				renderObject.RenderTargets[DST_ACCUMULATOR], 
				SCREEN_SIZE);
			
			//Flip the accumulator we are writing to
			swapTargets(SRC_ACCUMULATOR,DST_ACCUMULATOR);
			glFinish();			
			}
		cout<<"Bucket Pos: "<<bucketX<<endl;
		}

//Do Intensity normalization (based on number of samples on grating:
//	renderObject.Shaders[ACCUMULATE_SHADER]->assignUniform1i(ACCUMULATOR_TEXTURE2_UNIFORM,0);
	renderObject.Shaders[ACCUMULATE_SHADER]->assignUniform3f(ACCUMULATOR_COEFFS_UNIFORM,
										1.0f/(float)(g.getNumSamples()*g.getNumSamples()),0.0f,0.0f);								
	renderObject.Shaders[ACCUMULATE_SHADER]->assignUniform1i(ACCUMULATOR_FLAGS_UNIFORM,1);

	applyShader(renderObject.Shaders[ACCUMULATE_SHADER], renderObject.RenderTargets[SRC_ACCUMULATOR]->texture(),
				renderObject.RenderTargets[DST_ACCUMULATOR], SCREEN_SIZE);

	//here we reset the accumulator coefficients
	renderObject.Shaders[ACCUMULATE_SHADER]->assignUniform3f(ACCUMULATOR_COEFFS_UNIFORM, 1.0f,1.0f,0.0f);
	renderObject.Shaders[ACCUMULATE_SHADER]->assignUniform1i(ACCUMULATOR_FLAGS_UNIFORM,0);

	//Flip the accumulator we are writing to
	swapTargets(SRC_ACCUMULATOR,DST_ACCUMULATOR);	
	glFinish();

//Do the diffuse pass now:

	renderObject.RenderTargets[OUTPUT_BUFFER]->enable(); //Enable the buffer onto which we render temporary results
	glClearColor(0.0,0.0,0.0, 1);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	
	renderObject.Shaders[DIFFUSE_SHADER]->bind(); //Enable the diffuse shader

	//set the uniform variables for this shader
	renderObject.Shaders[DIFFUSE_SHADER]->setAllUniforms();

	renderObject.VertexBuffers[SCENE_VBO]->render(true);//bind and render the Scene geometry
	
	//finish up here (may not be needed inside the loop?)
	renderObject.Shaders[DIFFUSE_SHADER]->unbind(); //disable the WLF shader
	renderObject.RenderTargets[OUTPUT_BUFFER]->disable(); 

	applyShader(renderObject.Shaders[ACCUMULATE_SHADER], 
		renderObject.RenderTargets[SRC_ACCUMULATOR]->texture(), 
		renderObject.RenderTargets[OUTPUT_BUFFER]->texture(), 
		renderObject.RenderTargets[DST_ACCUMULATOR], 
		SCREEN_SIZE);

	//Flip the accumulator we are writing to
	swapTargets(SRC_ACCUMULATOR,DST_ACCUMULATOR);
	glFinish();
	
//save the result to an image	
	saveTextureToFile(renderObject.RenderTargets[SRC_ACCUMULATOR]->texture(), fname+".exr", SCREEN_SIZE);
	
//Do Tone Mapping:

	applyShader(renderObject.Shaders[TONE_MAP_SHADER], renderObject.RenderTargets[SRC_ACCUMULATOR]->texture(),
				renderObject.RenderTargets[DST_ACCUMULATOR], SCREEN_SIZE);

	//Flip the accumulator we are writing to
	swapTargets(SRC_ACCUMULATOR,DST_ACCUMULATOR);	
	glFinish();	
	
	//save the result to an image	
	saveTextureToFile(renderObject.RenderTargets[SRC_ACCUMULATOR]->texture(), fname, SCREEN_SIZE);
	
	return renderObject.RenderTargets[SRC_ACCUMULATOR]->texture();
	}

