#include <renderer.h>
#include <drawScene.h>
#include <vector>
using std::vector;

void createTiltedPlane(vector<vertexStruct>& vertices, vector<GLuint>& indices, float size,float depth, float tilt)
	{
		int offset = vertices.size();//how many vertices are already here?
	
		//FIXME: The normals are wrong 
		vertices.push_back(vertexStruct( -size, -size, depth, 0.0f, 0.0f, -1.0f, 0.0f, 0.0f ));
		vertices.push_back(vertexStruct( -size,  size, depth + 1.0f*tilt, 0.0f, 0.0f, -1.0f, 0.0f, 1.0f ));
		vertices.push_back(vertexStruct(  size,  size, depth + 2.0f*tilt, 0.0f, 0.0f, -1.0f, 1.0f, 1.0f ));
		vertices.push_back(vertexStruct(  size, -size, depth + 1.0f*tilt, 0.0f, 0.0f, -1.0f, 1.0f, 0.0f ));
		
		for (int i=0;i<4;i++)
			indices.push_back((GLuint)(offset+i));
	}

void createPlane(const Vector3f& center, const Vector3f& normal, const Vector3f& up, float size, vector<vertexStruct>& vertices, vector<GLuint>& indices)
	{
	int offset = vertices.size();//how many vertices are already here?
	Vector3f right = normal.cross(up).normalized();
	
	Vector3f pt = (up + right)*size + center;
	vertices.push_back(vertexStruct( pt.x(), pt.y(), pt.z(), normal.x(),normal.y(),normal.z(), 0.0f, 0.0f ));
	
	pt = (-up + right)*size + center;
	vertices.push_back(vertexStruct( pt.x(), pt.y(), pt.z(), normal.x(),normal.y(),normal.z(), 0.0f, 0.0f ));
	
	pt = (-up - right)*size + center;
	vertices.push_back(vertexStruct( pt.x(), pt.y(), pt.z(), normal.x(),normal.y(),normal.z(), 0.0f, 0.0f ));
	
	pt = (up - right)*size + center;
	vertices.push_back(vertexStruct( pt.x(), pt.y(), pt.z(), normal.x(),normal.y(),normal.z(), 0.0f, 0.0f ));		

	for (int i=0;i<4;i++)
		indices.push_back((GLuint)(offset+i));	
	}

void createBox(vector<vertexStruct>& vertices, vector<GLuint>& indices, float boxSize_by_2, const Vector3f& offset)
	{
//	createPlane(Vector3f(0,boxSize_by_2,0) + offset,Vector3f(0,1,0),Vector3f(0,0,1),boxSize_by_2,vertices,indices);
	createPlane(Vector3f(0,-boxSize_by_2,0) + offset,Vector3f(0,-1,0),Vector3f(0,0,1),boxSize_by_2,vertices,indices);
	
	createPlane(Vector3f(-boxSize_by_2,0,0) + offset,Vector3f(-1,0,0),Vector3f(0,-1,0),boxSize_by_2,vertices,indices);
//	createPlane(Vector3f(boxSize_by_2,0,0) + offset,Vector3f(1,0,0),Vector3f(0,-1,0),boxSize_by_2,vertices,indices);

//	createPlane(Vector3f(0,0,boxSize_by_2) + offset,Vector3f(0,0,1),Vector3f(0,1,0),boxSize_by_2,vertices,indices);
	createPlane(Vector3f(0,0,-boxSize_by_2) + offset,Vector3f(0,0,-1),Vector3f(0,1,0),boxSize_by_2,vertices,indices);		
	}

void createCornellRoom(vector<vertexStruct>& vertices, vector<GLuint>& indices, float boxSize_by_2, const Vector3f& offset)
	{

	createPlane(Vector3f(0,0,boxSize_by_2) + offset,Vector3f(0,0,-1),Vector3f(0,1,0),boxSize_by_2,vertices,indices);
//	createPlane(Vector3f(-boxSize_by_2,0,0) + offset,Vector3f(1,0,0),Vector3f(0,1,0),boxSize_by_2,vertices,indices);
	createPlane(Vector3f(boxSize_by_2,0,0) + offset,Vector3f(-1,0,0),Vector3f(0,1,0),boxSize_by_2,vertices,indices);		
	createPlane(Vector3f(0,boxSize_by_2,0) + offset,Vector3f(0,-1,0),Vector3f(0,0,-1),boxSize_by_2,vertices,indices);
//	createPlane(Vector3f(0,-boxSize_by_2,0) + offset,Vector3f(0,1,0),Vector3f(0,0,-1),boxSize_by_2,vertices,indices);

	}

void createScene(VertexBuffer* vbo)
	{
	//create some cpu buffers to store the vertex and index data
	vector<vertexStruct> vertices;
	vector<GLuint> indices;	
	
	//set the vbo parameters
	vbo->enableEntity(F_VERTEX, 3, 0);	
	vbo->enableEntity(F_NORMAL, 3, 3*sizeof(GLfloat));
	vbo->enableEntity(F_TEXTURE,2, 6*sizeof(GLfloat));
	
	vbo->enableEntity(F_INDEX, 1, 0);
	
	//construct geometry here (fill the vertices and indices vectors):
	//createTiltedPlane(vertices, indices,100,0,0);///size = -100 to +100, z=0, no tilt.
	createCornellRoom(vertices, indices,20, Vector3f(0,0,0));
	createBox(vertices, indices, 10, Vector3f(10,10,10));
	
	//createPlane(Vector3f(0,0,0),Vector3f(0,0,-1),Vector3f(0,1,0),5,vertices, indices);
	
	//initialize the vbo with all the geometry we have constructed
	vbo->init((const void*)&vertices[0],(GLuint *)&indices[0],sizeof(vertexStruct), vertices.size(), indices.size(), GL_QUADS);
	}

void drawQuad(float size)
	{			//render a quad 
		glBegin(GL_QUADS);
			{
			glTexCoord2f(0,0);	glVertex3f(0.0f ,0.0f,0.0f);
			glTexCoord2f(0,1);	glVertex3f(0.0f ,size,0.0f);
			glTexCoord2f(1,1);	glVertex3f(size ,size,0.0f);
			glTexCoord2f(1,0);	glVertex3f(size ,0.0f,0.0f);
			}
		glEnd();
	}

//void drawCornellRoom()
//	{
//		glBegin(GL_QUADS);
//			{
//			glTexCoord2f(0,0);	glVertex3f(	-10.0f,	-10.0f,	 10.0f );
//			glTexCoord2f(0,1);	glVertex3f(	-10.0f,	 10.0f,	 10.0f );
//			glTexCoord2f(1,1);	glVertex3f(	 10.0f,	 10.0f,	 10.0f );
//			glTexCoord2f(1,0);	glVertex3f(	 10.0f,	-10.0f,	 10.0f );
//			
////			glTexCoord2f(0,0);	glVertex3f(	-10.0f,	-10.0f,  10.0f );
////			glTexCoord2f(0,1);	glVertex3f(	-10.0f,	 10.0f,	 10.0f );
////			glTexCoord2f(1,1);	glVertex3f(	-10.0f,	 10.0f,	-10.0f );
////			glTexCoord2f(1,0);	glVertex3f(	-10.0f,	-10.0f,	-10.0f );
//			
//			glTexCoord2f(0,0);	glVertex3f(	 10.0f,	-10.0f,	 10.0f );
//			glTexCoord2f(1,0);	glVertex3f(	 10.0f,	-10.0f,	-10.0f );
//			glTexCoord2f(1,1);	glVertex3f(	 10.0f,	 10.0f,	-10.0f );			
//			glTexCoord2f(0,1);	glVertex3f(	 10.0f,	 10.0f,	 10.0f );

//			glTexCoord2f(0,0);	glVertex3f(  10.0f,	 10.0f,  10.0f );
//			glTexCoord2f(0,1);	glVertex3f(	-10.0f,	 10.0f,	 10.0f );
//			glTexCoord2f(1,1);	glVertex3f(	-10.0f,	 10.0f,	-10.0f );
//			glTexCoord2f(1,0);	glVertex3f(	 10.0f,	 10.0f,	-10.0f );
//			
//			glTexCoord2f(0,0);	glVertex3f(  10.0f,	-10.0f,  10.0f );
//			glTexCoord2f(1,0);	glVertex3f(	 10.0f,	-10.0f,	-10.0f );
//			glTexCoord2f(1,1);	glVertex3f(	-10.0f,	-10.0f,	-10.0f );			
//			glTexCoord2f(0,1);	glVertex3f(	-10.0f,	-10.0f,	 10.0f );

//			}
//		glEnd();
//	}
	


//void drawMesh()
//	{
//	// Draw the geometry

//	const float resolution=4;
//	const float x1=0,y1=0, step=5;

//	const float smallStep=step/resolution;
//	glBegin(GL_TRIANGLES);
//	for (int i = 0; i < resolution; i++)
//		for (int j = 0; j < resolution; j++)
//		{
//		glTexCoord2f(i*smallStep,j*smallStep);		glVertex2f(x1+i*step,y1+j*step);
//		glTexCoord2f((i+1)*smallStep,j*smallStep);	glVertex2f(x1+(i+1)*step,y1+j*step);
//		glTexCoord2f(i*smallStep,(j+1)*smallStep);	glVertex2f(x1+i*step,y1+(j+1)*step);
//	
//		glTexCoord2f(i*smallStep,(j+1)*smallStep);	glVertex2f(x1+i*step,y1+(j+1)*step);
//		glTexCoord2f((i+1)*smallStep,j*smallStep);	glVertex2f(x1+(i+1)*step,y1+j*step);			
//		glTexCoord2f((i+1)*smallStep,(j+1)*smallStep);	glVertex2f(x1+(i+1)*step,y1+(j+1)*step);			
//		}
//	glEnd();
//	}
