uniform sampler2D sourceTexture; //the source image that we are tone mapping

uniform float defog;
uniform float exposure;
uniform float knee_low;
uniform float knee_f_factor;
const float gamma=0.5;//2.2 hack by kashyap;//FIXME:Make this a uniform

vec4 apply_defog(vec4 v)
    {
    //
    // Defog
    //
    vec4 retval;
    retval.x=max (0.0, (v.x - defog));
    retval.y=max (0.0, (v.y - defog));
    retval.z=max (0.0, (v.z - defog));
    retval.w=max (0.0, (v.w - defog));        
    return retval;
    }
    
vec4 apply_exposure(vec4 v)
    {
    //
    // Exposure
    //
    return v * exp2(exposure + 2.47393);
    }

float knee(float x)
    {
    return log(x * knee_f_factor + 1.0) / knee_f_factor;
    }
    
float apply_knee(float x)
    {
    //
    // Knee
    //
    if (x > knee_low)
	    return knee_low + knee (x - knee_low);//, knee_f_factor);modified by kashyap
  	else
	    return x;
    }

vec4 apply_knee4(vec4 v)
	{
	return vec4(apply_knee(v.x),apply_knee(v.y),apply_knee(v.z),apply_knee(v.w));
	}

vec4 apply_gamma(vec4 v)
    {
    //
    // Gamma
    //
    return vec4(pow (v.x, gamma), pow(v.y,gamma), pow(v.z,gamma), pow(v.w,gamma));
    }
    
vec4 apply_scaling(vec4 v)   
    {
    //
    // Scale 
    //
    return v * exp2(-3.5 * gamma);
    }

void main()
	{
	vec4 input_value=texture2D(sourceTexture, gl_TexCoord[0].xy);
	vec4 temp=apply_defog(input_value);//defogged
	temp=apply_exposure(temp);//applied exposure
	temp=apply_knee4(temp);//applied knee
	temp=apply_gamma(temp);//gamma corrected
	gl_FragColor=apply_scaling(temp);//applied scaling
	}

