const float PI = 3.141593;

//NOTE: All these variables to be made uniforms and passed to shader

//const float r_lambda=720e-6;
//const float g_lambda=530e-6;
//const float b_lambda=470e-6;

const float lambda1	= 400e-6;//b-
const float lambda2	= 450e-6;//b
const float lambda3	= 500e-6;//b+
const float lambda4	= 550e-6;//g
const float lambda5	= 600e-6;//g+
const float lambda6	= 650e-6;//r
const float lambda7	= 700e-6;//r+

uniform vec3 lightSource;//light source 2 meters behind grating
uniform vec4 lightColor;//brightness and color of the light source

//These 3 params specify the position and orientation of the grating.
uniform vec3 gratingPosition;
uniform vec3 gratingXAxis; //x axis
uniform vec3 gratingYAxis; //y axis

//This defines the sampling on the grating: smaller is better
uniform float sampleStep;

//defines the bucket start
uniform vec2 bucketStart;//position of bucket

uniform float gratingSize;//total size of the grating
uniform int randomSize;//size of random texture
uniform int bucketSize;//number of sampleSteps in a bucket

////multiply the final intensity by this number.
//uniform float intensityNorm;

//input light filter
uniform sampler2D lightFilterTexture;

//random texture
uniform sampler2D randomTexture;

//Current Pixel's position, set by vertex shader
varying vec3 position;
//Current Pixel's normal, set by vertex shader
varying vec3 surface_normal;

const float EPSILON_FLOAT1 = 0.01;
const float EPSILON_FLOAT2 = 0.0001;

//this function is defined in a different shader file, and returns the wdf value
float wdf_wrapper(float x,float y,float u,float v, float lambda);

float kdelta(float x){
	if (abs(x)<EPSILON_FLOAT1)
		return 1.0;
	else
		return 0.0;
}

float triangleFunction(float x){
	return max(0.0,1.0-abs(x));
}

float sinc(float x){
	if (abs(x)<EPSILON_FLOAT2)
		return cos(x*PI);
	else
		return sin(x*PI)/(x*PI);
}

float rect(float x){
	if (abs(x)<1.0)
		return 1.0;
	else
		return 0.0;
}

void main()
{
//ALL UNITS ARE IN MilliMeters

	//storage for the 7 color trick
	float color1=0.0,color2=0.0,color3=0.0,color4=0.0,color5=0.0,color6=0.0,color7=0.0;
	
	vec3 gratingNormal = cross(gratingXAxis, gratingYAxis); //calcuate the normal to the grating.
	
	float dotProduct = -dot(gratingNormal,surface_normal);
	
	if (dotProduct < 0){
		discard;
	}	
	
	float bucketWidth = float(bucketSize) * sampleStep; //size of bucket in mm
	
	float inv_gratingSize = 1.0/gratingSize;
	
	//to access random texture, (we tile the random texture over the grating)
	float randomIncrement = 1.0/float(randomSize);	
	float randomCoordYStart= (gratingSize/sampleStep/float(randomSize))*((bucketStart.y*inv_gratingSize)+0.5);	
	vec2 randomCoord=vec2(	(gratingSize/sampleStep/float(randomSize))*((bucketStart.x*inv_gratingSize)+0.5),
							randomCoordYStart);
														
	for (float i = bucketStart.x; i < bucketStart.x + bucketWidth; i += sampleStep){
		
		randomCoord.y = randomCoordYStart;
		randomCoord.x += randomIncrement;
		
		for (float j = bucketStart.y; j < bucketStart.y + bucketWidth; j += sampleStep){
		
			randomCoord.y += randomIncrement;
			vec4 rand = 2.0 * texture2D(randomTexture,randomCoord) - 1.0;//  vec4(0,0,0,0);//
			
			//Note that filter color is a vec4, but still we only use the X component (red) for intensity.
			//So ensure that all components of the texture have same values.
			vec4 filterColor = texture2D(lightFilterTexture, vec2(i*inv_gratingSize+0.5, j*inv_gratingSize+0.5));

			vec3 currentGratingPos =	gratingPosition + 
										gratingXAxis*(i + rand.r*sampleStep ) + 
										gratingYAxis*(j + rand.g*sampleStep );

			vec3 ray, tmpRay;

		//ray coming from light source to grating
			ray = normalize(currentGratingPos - lightSource);

		//project ray onto the x-z plane
			tmpRay = normalize(ray - gratingYAxis * ray.y);	
		//get input angle in x-z plane:
			float theta1 = asin(dot(cross(gratingNormal, tmpRay), gratingYAxis));

		//project ray onto the y-z plane
			tmpRay = normalize(ray - gratingXAxis * ray.x);
		//get input angle in y-z plane:
			float phi1   = asin(dot(cross(gratingNormal, tmpRay), gratingXAxis));

		//outgoing ray:
			ray = normalize(position - currentGratingPos);
			
		//project ray onto the x-z plane
			tmpRay = normalize(ray - gratingYAxis * ray.y);
		//get output angle in x-z plane:
			float theta2 = asin(dot(cross(gratingNormal, tmpRay), gratingYAxis));

		//project ray onto the y-z plane
			tmpRay = normalize(ray - gratingXAxis * ray.x);
		//get output angle in y-z plane:
			float phi2   = asin(dot(cross(gratingNormal, tmpRay), gratingXAxis));

		//relative deflection:
			float theta = theta2- theta1;
			float phi   = phi2- phi1;
		
		//The 7 color trick
			color1 +=  wdf_wrapper(i,j, theta/lambda1,phi/lambda1, lambda1);//*filterColor.x;
			color2 +=  wdf_wrapper(i,j, theta/lambda2,phi/lambda2, lambda2);//*filterColor.x;
			color3 +=  wdf_wrapper(i,j, theta/lambda3,phi/lambda3, lambda3);//*filterColor.x;
			color4 +=  wdf_wrapper(i,j, theta/lambda4,phi/lambda4, lambda4);//*filterColor.x;
			color5 +=  wdf_wrapper(i,j, theta/lambda5,phi/lambda5, lambda5);//*filterColor.x;
			color6 +=  wdf_wrapper(i,j, theta/lambda6,phi/lambda6, lambda6);//*filterColor.x;
			color7 +=  wdf_wrapper(i,j, theta/lambda7,phi/lambda7, lambda7);//*filterColor.x;

		//do any color blending etc here...
//			colorFinal.r +=  wdf_2DBinaryGrating(i,j, theta/r_lambda,phi/r_lambda, 0.1,0.5);//pitch=0.1,dutyCycle=0.5
//			colorFinal.g +=  wdf_2DBinaryGrating(i,j, theta/g_lambda,phi/g_lambda, 0.1,0.5);		
//			colorFinal.b +=  wdf_2DBinaryGrating(i,j, theta/b_lambda,phi/b_lambda, 0.1,0.5);										
		}
	}
	//finally, multiply by lightsource color  (tone mapping later...)
	
//	gl_FragColor = vec4(color7,0.0,0.0,1.0)*lightColor * dotProduct;//one color trick ;P

	//again, 7 color blending trick
	gl_FragColor =  vec4(color7*0.25 + color6*0.5 + color5*0.25,
					color5*0.25 + color4*0.5 + color3*0.25,
					color3*0.25 + color2*0.5 + color1*0.25, 1.0) * lightColor * dotProduct;

	//normalize the intensity 
//	gl_FragColor *= intensityNorm;
}

////////////////////SOME OLD CODE/////////////////////////


//float gratingEquation(const float x,const float u,const float lambda,const float m,const float p){
//	float u2pi=2.0*PI*u;
//	float theta=u2pi*lambda;
//	float lambda_by_p=lambda/p;
//	
//	return 	0.25*( (1.0 + m*m/2.0*cos(4.0*PI*x/p))*kdelta(theta) +
//			m*cos(2.0*PI*x/p)*(kdelta(theta-lambda_by_p/2.0)+kdelta(theta+lambda_by_p/2.0)) +
//			m*m/4.0*(kdelta(theta-lambda_by_p) +kdelta(theta+lambda_by_p)));
//}

//float wdf_2DCosineGrating(float x,float y,float u,float v,float lambda){
//	const float m=1.0, p=100.0;
//	float tmpX=	gratingEquation(x,u,lambda,m,p);
//	float tmpY= gratingEquation(y,v,lambda,m,p);
//	return tmpX*tmpY;
//}
