const float PI = 3.141593;

float rect(float x);
float sinc(float x);
float triangleFunction(float x);
float kdelta(float x);

//generic uniforms to control grating parameters
uniform vec4 gratingParameters;

float apertureGrating(float x, float u, float pitch, float dutyCycle, float aperture){
	const int terms=4;
	float result = 0.0;
	for (int q1 = -terms; q1<=terms; q1++){	
		for (int q2 = -terms; q2<=terms; q2++){
			result += 	sinc(dutyCycle * float(q1)) *  
						sinc(dutyCycle * float(q2)) * 
						cos(2.0 * PI * float(q1-q2) * x / pitch);
		}
	}
	return 	(2.0*aperture - 4.0*abs(x)) * rect(x/aperture) * 
			sinc((2.0*aperture - 4.0*abs(x))*u) * 
			(dutyCycle * dutyCycle) * result;
}

float wdf_wrapper(float x,float y,float u,float v, float lambda){
//the meaning of each component of gratingParameters is defined here:

//Square Aperture * Binary Grating:
	//gratingParameters.x => grating pitch
	//gratingParameters.y => grating duty cycle
	//gratingParameters.z => aperture width
	return	apertureGrating(x,u,gratingParameters.x,gratingParameters.y, gratingParameters.z) *
			apertureGrating(y,v,gratingParameters.x,gratingParameters.y, gratingParameters.z);

}
