const float PI = 3.141593;

float rect(float x);
float sinc(float x);
float triangleFunction(float x);
float kdelta(float x);

//generic uniforms to control grating parameters
uniform vec4 gratingParameters;

float binaryGrating(float x, float u, float pitch, float dutyCycle){
	const int terms=4;
	float result = 0.0;
	for (int q1 = -terms; q1<=terms; q1++){	
		for (int q2 = -terms; q2<=terms; q2++){
			if (kdelta(u - float(q1+q2)/(2.0*pitch))>0.0)
				result += 	sinc(dutyCycle * float(q1)) *  
							sinc(dutyCycle * float(q2)) * 
							cos(2.0 * PI * float(q1-q2) * x / pitch);						
		}
	}
	return (dutyCycle * dutyCycle) * result; 
}

float wdf_2DBinaryGrating(float x,float y,float u,float v,float pitch, float dutyCycle){
	return binaryGrating(x,u,pitch,dutyCycle) * binaryGrating(y,v,pitch,dutyCycle);
}

float wdf_wrapper(float x,float y,float u,float v, float lambda){
//the meaning of each component of gratingParameters is defined here:

//Binary Grating:
	//gratingParameters.x => grating pitch
	//gratingParameters.y => grating duty cycle
	return	binaryGrating(x,u,gratingParameters.x,gratingParameters.y) *
			binaryGrating(y,v,gratingParameters.x,gratingParameters.y);
}
