#include<program.h>
#include<grating.h>
#include<Eigen/Geometry>

const int X__=0;
const int Y__=1;
const int Z__=2;

const int GratingInitXform__=0;
const int GratingMoveXform__=1;

void grating::setGratingDimensions(float gratingSize, int bucketSize, int numSamples)
	{
	this->gratingSize = gratingSize;
	this->bucketSize = bucketSize;
	this->numSamples = numSamples;
	}

float grating::getGratingSize() const
	{
	return gratingSize;
	}
int grating::getBucketSize() const
	{
	return bucketSize;
	}
int grating::getNumSamples() const
	{
	return numSamples;
	}

void grating::bindToShader(ProgramGLSL * shader,int gratingSizeLoc, int posnLoc, int xLoc, int yLoc, int bucketSizeLoc, 
							int sampleStepLoc, int lightPosnLoc, int lightColorLoc)
	{
	wlf_shader = shader;
	LIGHTSOURCE_UNIFORM = lightPosnLoc;
	LIGHTCOLOR_UNIFORM = lightColorLoc;
	GRATING_POSITION_UNIFORM = posnLoc;
	GRATING_XAXIS_UNIFORM = xLoc;
	GRATING_YAXIS_UNIFORM = yLoc;
	BUCKET_SIZE_UNIFORM = bucketSizeLoc;
	SAMPLE_STEP_UNIFORM = sampleStepLoc;
	GRATING_SIZE_UNIFORM = gratingSizeLoc;
	}

void grating::assignGratingParams()const
    {
    wlf_shader->assignUniform3f(GRATING_POSITION_UNIFORM,
							gratingCentre.x(),
							gratingCentre.y(),
							gratingCentre.z());
    wlf_shader->assignUniform3f(GRATING_XAXIS_UNIFORM,
							gratingAxes[X__].x(),
							gratingAxes[X__].y(),
							gratingAxes[X__].z());
    wlf_shader->assignUniform3f(GRATING_YAXIS_UNIFORM,
							gratingAxes[Y__].x(),
							gratingAxes[Y__].y(),
							gratingAxes[Y__].z());

	wlf_shader->assignUniform1i(BUCKET_SIZE_UNIFORM, bucketSize);
	wlf_shader->assignUniform1f(GRATING_SIZE_UNIFORM, gratingSize);
	wlf_shader->assignUniform1f(SAMPLE_STEP_UNIFORM, gratingSize/(float)numSamples);

	//lets not update uniforms in shader here, let the rendering function bother about it.
	//wlf_shader->setAllUniforms(true);//inefficient, i know, but in the name of abstraction :)
    }

void grating::setLightColor(float r, float g, float b)
	{
	lightColor = Vector4f(r,g,b,1.0f);
	wlf_shader->assignUniform4f(LIGHTCOLOR_UNIFORM,
							lightColor.x(),
							lightColor.y(),
							lightColor.z(),
							lightColor.w());	
	//wlf_shader->setAllUniforms(true);
	}
	
void grating::setLightPosn(float x, float y, float z)
	{
	lightPosn=Vector3f(x,y,z);
	wlf_shader->assignUniform3f(LIGHTSOURCE_UNIFORM,
							lightPosn.x(),
							lightPosn.y(),
							lightPosn.z());	
	//wlf_shader->setAllUniforms(true);
	}
	
inline void grating::setMovement(const RigidTransformF& other)
	{
	gratingTransform[GratingMoveXform__]=other;
	}
	
void grating::update(float t)
	{
	if ( (t <0.0f) || (t>1.0f) )
	    return;
	RigidTransformF temp;//relying on the default constructor to initialize temp to identity transformation
	temp=temp*(1.0f-t) + gratingTransform[GratingMoveXform__]*(t);
	temp.normalize();
	temp=temp*gratingTransform[GratingInitXform__];

	Vector3f TransVec=temp.translation();
	Matrix3f RotMat=temp.rotation();
	
	gratingCentre=TransVec;
	gratingAxes[X__]=RotMat.col(X__)+TransVec;
	gratingAxes[Y__]=RotMat.col(Y__)+TransVec;
	gratingAxes[Z__]=RotMat.col(Z__)+TransVec;
	
	assignGratingParams();//update params in shader 
	}

void grating::checkPoint(float t)
	{
	if ( (t <0.0f) || (t>1.0f) )
	    return;
	RigidTransformF temp;//relying on the default constructor to initialize temp to identity transformation
	temp=temp*(1.0f-t) + gratingTransform[GratingMoveXform__]*(t);
	temp.normalize();
	gratingTransform[GratingInitXform__]=temp*gratingTransform[GratingInitXform__];

	Vector3f TransVec=gratingTransform[GratingInitXform__].translation();
	Matrix3f RotMat=gratingTransform[GratingInitXform__].rotation();
	
	gratingCentre=TransVec;
	gratingAxes[X__]=RotMat.col(X__)+TransVec;
	gratingAxes[Y__]=RotMat.col(Y__)+TransVec;
	gratingAxes[Z__]=RotMat.col(Z__)+TransVec;
	
	assignGratingParams(); //update params in shader
	}
	
void grating::lookat(const Vector3f &lookat, const Vector3f &center, const Vector3f &up)
    {
    gratingAxes[Y__]=up;
    gratingAxes[Z__]=(lookat-center); 
    gratingAxes[Y__].normalize();
    gratingAxes[Z__].normalize();  
    gratingAxes[X__]=gratingAxes[Y__].cross(gratingAxes[Z__]);
    gratingAxes[X__].normalize();
    //modifications by kashyap (to get true up vector)
    gratingAxes[Y__]=gratingAxes[Z__].cross(gratingAxes[X__]);
    gratingAxes[Y__].normalize();
    gratingCentre = center;
    //end of modifications
    Matrix3f sceneToGrating;
    sceneToGrating.col(X__)=gratingAxes[X__];
    sceneToGrating.col(Y__)=gratingAxes[Y__];
    sceneToGrating.col(Z__)=gratingAxes[Z__];
    Eigen::Quaternionf primQuat(sceneToGrating);
    Eigen::Quaternionf dualQuat(0, center.x()*0.5f, center.y()*0.5f, center.z()*0.5);
    gratingTransform[GratingInitXform__]=RigidTransformF(primQuat, dualQuat*primQuat);
    
   	assignGratingParams(); //update params in shader
    }
