#ifndef CAMERA_H
#define CAMERA_H

#include <GL/glew.h>
#include "vectorMath.h"
#include <cmath>
using namespace std;

enum DIRECTION{FORWARD=0,RIGHT=1,UP=2};

class quaternion{
	float u;
	vector3d v;
public:
	quaternion(){
	}
	quaternion(float angle, vector3d axis){
		init(angle, axis);
	}
	void normalize(){
		float scaleFactor=1.0/(u*u+vector3d::dotProduct(v,v));
		u=u*scaleFactor;
		v=v.scale(scaleFactor);
		//cout<<"Quat: "<<u<<","<<v.x<<","<<v.y<<","<<v.z<<endl;
	}
	void init(const float angle,const vector3d& axis){
		u=cos(angle/2);
		v=axis.scale(sin(angle/2));
		normalize();
	}
	quaternion inverse() const{
		quaternion temp;
		temp.u=u;
		temp.v=v.negative();		
		temp.normalize();
		return temp;
	}
	static quaternion multiply(const quaternion& a,const quaternion& b){
		quaternion temp;
		temp.u=a.u*b.u - vector3d::dotProduct(a.v,b.v);
		temp.v=a.v.scale(b.u) + b.v.scale(a.u) + vector3d::crossProduct(a.v,b.v);
		return temp;
	}
	vector3d rotate(vector3d p) const{
		quaternion p_quat;
		p_quat.u=0; p_quat.v=p;
		quaternion retval=multiply(multiply(*this,p_quat),inverse());
		return retval.v;
	}
};

class Camera{
	vector3d lookAt, camPos, upVector;//camera vectors

	void setupOpenGLCamera(){
		glMatrixMode(GL_MODELVIEW); 
		glLoadIdentity();
		gluLookAt(	//r*sin(theta)*cos(phi), r*sin(theta)*sin(phi), r*cos(theta),
					camPos.x,camPos.y,camPos.z,
					lookAt.x,lookAt.y,lookAt.z, 
					upVector.x,upVector.y,upVector.z
				);			
	}

public:
	
	vector3d getEye(){
		return camPos;
	}
	
	void init(float x, float y, float z, float cx,float cy, float cz, float ux, float uy, float uz){
		camPos=vector3d(x,y,z);
		lookAt=vector3d(cx,cy,cz);
		upVector=vector3d(ux,uy,uz);
		
		setupOpenGLCamera();
	}

	void moveCamera(int axis, float delta){		
		vector3d move;
		switch(axis){
			case FORWARD://move in direction of view vector
				move=(lookAt-camPos).getUnitVector().scale(delta);
				break;
			case RIGHT:
				move=vector3d::crossProduct(lookAt-camPos,upVector).getUnitVector().scale(delta);
				break;
			case UP:
				move=upVector.scale(delta);
				break;
		}
		camPos=camPos + move;
		lookAt=lookAt + move;		
		setupOpenGLCamera();
	}
	
	void rotateCamera(int axis, float delta){
	//axis  parameter: 0->about up vector, 1->about right vector
		quaternion q;
		switch(axis){
			case UP:
				q.init(delta,upVector);
				lookAt=(camPos+q.rotate(lookAt-camPos));
				break;
			case RIGHT:
				vector3d rightVector=vector3d::crossProduct(lookAt-camPos,upVector).getUnitVector();
				q.init(delta,rightVector);
				lookAt=(camPos+q.rotate(lookAt-camPos));
				//upVector=vector3d::crossProduct(rightVector,lookAt-camPos).getUnitVector();
				break;
		}
		setupOpenGLCamera();
	}
};


#endif
