#ifndef GRATING_H
#define GRATING_H

#include<RigidTransform.h>
#include<Eigen/Core>
#include<Eigen/Geometry>
#include<program.h>
using namespace Eigen;
//essential definitions for our object manipulation
typedef RigidTransform<float> RigidTransformF;  	       
typedef Translation<float,3> Xlate;
typedef AngleAxis<float> Rotate;

class grating
	{
public:
	EIGEN_MAKE_ALIGNED_OPERATOR_NEW
	    
	void setLightPosn(float r, float g, float b);
	void setLightColor(float x, float y, float z);
	
	void setMovement(const RigidTransformF& other);
	void update(const float t);
	void checkPoint(const float t);
	
	//binds this object to the shader, by setting some uniforms
	void bindToShader(ProgramGLSL * shader,int gratingSizeLoc, int posnLoc, int xLoc, int yLoc, int bucketSizeLoc, 
							int sampleStepLoc, int lightPosnLoc, int lightColorLoc);

	void lookat(const Vector3f &lookat, const Vector3f &center, const Vector3f &up);
	
	void setGratingDimensions(float gratingSize, int bucketSize, int numSamples);
	float getGratingSize() const;
	int getBucketSize() const;
	int getNumSamples() const;
	
protected:	
	void assignGratingParams()const;//updates uniforms for grating position
	
	RigidTransformF gratingTransform[2];
	
	Vector3f gratingAxes[3];
	Vector3f gratingCentre;
	
	Vector4f lightColor;
	Vector3f lightPosn;
	
	ProgramGLSL * wlf_shader;
	
	float gratingSize;
	int numSamples, bucketSize;
	
	int LIGHTSOURCE_UNIFORM, LIGHTCOLOR_UNIFORM, 
		GRATING_POSITION_UNIFORM, GRATING_XAXIS_UNIFORM, 
		GRATING_YAXIS_UNIFORM,BUCKET_SIZE_UNIFORM,
		GRATING_SIZE_UNIFORM,SAMPLE_STEP_UNIFORM;
	};
#endif
