/*
 * Copyright (C) 2008 Rivo Laks <rivolaks@hot.ee>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either 
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public 
 * License along with this program.  If not, see <http://www.gnu.org/licenses/>.

 */
#include<iostream>
#include<cstdio>
#include"shader.h"
using namespace std;


Shader::Shader(GLenum type)
{
    init(type);
}

Shader::Shader(const GLenum type,const string& filename)
{
    init(type);

    FILE *shaderFile;
    char *text;
    long size;
    size_t temp;

    //copy name of the file
    mFileName=filename;

    //must read files as binary to prevent problems from newline translation
    shaderFile = fopen( filename.c_str(), "rb");

    if ( shaderFile == NULL)
	{
        cerr << "Shader::Shader(QString): Can't open file" << filename << "for reading";
        return ;
	}

    fseek( shaderFile, 0, SEEK_END);

    size = ftell(shaderFile);

    fseek( shaderFile, 0, SEEK_SET);

    text = new char[size+1];

    temp=fread( text, size, 1, shaderFile);

    fclose( shaderFile);

    text[size] = '\0';

//    GLuint object = CompileGLSLShader( target, text);

    setSource(text);
    compile();
}

Shader::~Shader()
{
    glDeleteShader(glId());
    delete[] mCompileLog;
}

void Shader::init(GLenum type)
{
    mType = type;
    mValid = false;
    mCompiled = false;
    mCompileLog = false;
    mGLId = glCreateShader(mType);
}

void Shader::setSource(const char * source)
{
    if (isCompiled()) {
        cerr << "Shader::setSource(): Can't set source for compiled shaders";
        return;
    }
//    const char* src = source.c_str();
    glShaderSource(glId(), 1, &source, NULL);
}

bool Shader::compile()
{
    if (isCompiled()) {
        cerr << "Shader::compile(): Can't compile a shader twice";
        return false;
    }
    // Compile the shader
    glCompileShader(glId());
    mCompiled = true;
    // Make sure it compiled correctly
    int compiled;
    glGetShaderiv(glId(), GL_COMPILE_STATUS, &compiled);
    mValid = compiled;
    // Get info log
    GLsizei logsize, logarraysize;
    glGetShaderiv(glId(), GL_INFO_LOG_LENGTH, &logarraysize);
    mCompileLog = new char[logarraysize];
    glGetShaderInfoLog(glId(), logarraysize, &logsize, mCompileLog);
    // Output the log if compilation failed
    if (!mValid) {
        cerr << "Shader::compile(): Couldn't compile "<<mFileName<<". Log follows:" << endl << mCompileLog;
    }
    if (!logsize) {
        delete[] mCompileLog;
        mCompileLog = NULL;
    }
    return mValid;
}

VertexShader::VertexShader() : Shader(GL_VERTEX_SHADER)
{
}

VertexShader::VertexShader(const char* filename) : Shader(GL_VERTEX_SHADER, filename)
{
}

FragmentShader::FragmentShader() : Shader(GL_FRAGMENT_SHADER)
{
}

FragmentShader::FragmentShader(const char * filename) : Shader(GL_FRAGMENT_SHADER, filename)
{
}
