#include <vectorMath.h>
#include <math.h>

vector3d::vector3d(float x, float y, float z){
	this->x=x;
	this->y=y;
	this->z=z;
}

vector3d vector3d::scale(float factor) const{
	vector3d tmp;
	tmp.x=x*factor;
	tmp.y=y*factor;
	tmp.z=z*factor;
	return tmp;
}
void vector3d::setXYZ(float x, float y, float z){
	this->x=x;
	this->y=y;
	this->z=z;
}
float vector3d::absolute() const{
	return sqrt(x*x+y*y+z*z);
}
vector3d vector3d::getUnitVector() const{
	float abVect=absolute();
	vector3d retVal=*this;
	if (abVect>0)
		retVal=retVal.scale(1/abVect);
	else
		retVal.setXYZ(0,0,0);

	return retVal;
}

float vector3d::dotProduct(const vector3d & v1,const vector3d & v2){
	return (v1.x*v2.x + v1.y*v2.y + v1.z*v2.z);
}

vector3d vector3d::crossProduct(const vector3d & v1,const vector3d & v2){
	vector3d retVal;
	retVal.x = v1.y * v2.z - v1.z * v2.y;
    retVal.y = v1.z * v2.x - v1.x * v2.z;
    retVal.z = v1.x * v2.y - v1.y * v2.x;
	return retVal;
}

vector3d vector3d::operator - (const vector3d & v) const{
	vector3d retVal;
	retVal.x=x-v.x;
	retVal.y=y-v.y;
	retVal.z=z-v.z;
	return retVal;
}

vector3d vector3d::operator + (const vector3d & v) const{
	vector3d retVal;
	retVal.x=x+v.x;
	retVal.y=y+v.y;
	retVal.z=z+v.z;
	return retVal;
}

bool vector3d::operator == (const vector3d & v) const{
	return  (x==v.x && y==v.y && z==v.z);
}

vector3d vector3d::parallelTo(const vector3d & ref) const{
	vector3d retVal;
	retVal=ref.getUnitVector();
	retVal=retVal.scale(vector3d::dotProduct(*this,ref));
	return retVal;
}

vector3d vector3d::perpendicularTo(const vector3d & ref) const{
	vector3d retVal;
	vector3d tmp1=*this;
	vector3d tmp2=parallelTo(ref);
	retVal=tmp1-tmp2;
	return retVal;
}

float vector3d::distanceTo(const vector3d & v) const{
	return ((*this-v).absolute());
}

bool vector3d::isNullVector() const{
	return (x==0 && y==0 && z==0);
}

bool vector3d::sameDirection(const vector3d v) const{
	vector3d tmp1,tmp2;
	tmp1=this->getUnitVector();
	tmp2=v.getUnitVector();
	return (tmp1==tmp2);
}

vector3d vector3d::negative() const{
	vector3d tmp;
	tmp.x=-x;
	tmp.y=-y;
	tmp.z=-z;
	return tmp;
}
